/*
NSEvent.h
An event generated by the user, such as a mouse movement or key press.
Copyright (c) 1994, NeXT, Inc.  All rights reserved.
*/

#import <Foundation/NSObject.h>
#import "dpsOpenStep.h"
#import "NSDPSContext.h"
@class NSWindow;

typedef enum _NSEventType {		/* various types of events */
    NSLeftMouseDown		= 1,
    NSLeftMouseUp		= 2,
    NSRightMouseDown		= 3,
    NSRightMouseUp		= 4,
    NSMouseMoved		= 5,
    NSLeftMouseDragged		= 6,
    NSRightMouseDragged		= 7,
    NSMouseEntered		= 8,
    NSMouseExited		= 9,
    NSKeyDown			= 10,
    NSKeyUp			= 11,
    NSFlagsChanged		= 12,
#ifndef STRICT_OPENSTEP
    NSAppKitDefined		= 13,
    NSSystemDefined		= 14,
    NSApplicationDefined	= 15,
#endif
    NSPeriodic			= 16,
    NSCursorUpdate		= 17
} NSEventType;

enum {					/* masks for the types of events */
    NSLeftMouseDownMask		= 1 << NSLeftMouseDown,
    NSLeftMouseUpMask		= 1 << NSLeftMouseUp,
    NSRightMouseDownMask	= 1 << NSRightMouseDown,
    NSRightMouseUpMask		= 1 << NSRightMouseUp,
    NSMouseMovedMask		= 1 << NSMouseMoved,
    NSLeftMouseDraggedMask	= 1 << NSLeftMouseDragged,
    NSRightMouseDraggedMask	= 1 << NSRightMouseDragged,
    NSMouseEnteredMask		= 1 << NSMouseEntered,
    NSMouseExitedMask		= 1 << NSMouseExited,
    NSKeyDownMask		= 1 << NSKeyDown,
    NSKeyUpMask			= 1 << NSKeyUp,
    NSFlagsChangedMask		= 1 << NSFlagsChanged,
#ifndef STRICT_OPENSTEP
    NSAppKitDefinedMask		= 1 << NSAppKitDefined,
    NSSystemDefinedMask		= 1 << NSSystemDefined,
    NSApplicationDefinedMask	= 1 << NSApplicationDefined,
#endif
    NSPeriodicMask		= 1 << NSPeriodic,
    NSCursorUpdateMask		= 1 << NSCursorUpdate,
    NSAnyEventMask		= 0xffffffff
};

static inline unsigned int NSEventMaskFromType(NSEventType type) { return (1 << type); }

/* Device-independent bits found in event modifier flags */
enum {
    NSAlphaShiftKeyMask =	1 << 16,
    NSShiftKeyMask =		1 << 17,
    NSControlKeyMask =		1 << 18,
    NSAlternateKeyMask =	1 << 19,
    NSCommandKeyMask =		1 << 20,
    NSNumericPadKeyMask =	1 << 21,
    NSHelpKeyMask =		1 << 22,
    NSFunctionKeyMask =		1 << 23
};

@interface NSEvent : NSObject {
    NSEventType _type;
    NSPoint _location;
    unsigned int _modifierFlags;
    int _WSTimestamp;
    NSTimeInterval _timestamp;
    int _windowNumber;
    NSWindow *_window;
    NSDPSContext *_context;
    union {
        struct {
            int eventNumber;
            int	clickCount;
            float pressure;
        } mouse;
        struct {
            NSString *keys;
            NSString *unmodKeys;
            unsigned short keyCode;
            BOOL isARepeat;
        } key;
        struct {
            int eventNumber;
            int	trackingNumber;
            int	userData;
        } tracking;
        struct {
            int subtype;
            int data1;
            int data2;
        } misc;
    } _data;
}


/* these messages are valid for all events */
- (NSEventType)type;
- (NSPoint)locationInWindow;
- (unsigned int)modifierFlags;
- (NSTimeInterval)timestamp;
- (NSWindow *)window;
- (int)windowNumber;
- (NSDPSContext *)context;

    /* these messages are valid for all mouse down/up/drag events */
- (int)clickCount;
- (float)pressure;
    /* these messages are valid for all mouse down/up/drag and enter/exit events */
- (int)eventNumber;

    /* these messages are valid for keyup and keydown events */
- (NSString *)characters;
- (NSString *)charactersIgnoringModifiers;
    /* the chars that would have been generated, regardless of modifier keys (except shift) */
- (BOOL)isARepeat;
    /* this message is valid for keyup, keydown and flagschanged events */
- (unsigned short)keyCode;		/* device-dependent key number */

    /* these messages are valid for enter and exit events */
- (int)trackingNumber;
- (int)userData;

    /* these messages are valid for kit, system, and app-defined events */
- (short)subtype;
- (int)data1;
- (int)data2;

    /* used for initial delay and periodic behavior in tracking loops */
+ (void)startPeriodicEventsAfterDelay:(NSTimeInterval)delay withPeriod:(NSTimeInterval)period;
+ (void)stopPeriodicEvents;

    /* apps will rarely create these objects */
+ (NSEvent *)mouseEventWithType:(NSEventType)type location:(NSPoint)location modifierFlags:(unsigned int)flags timestamp:(NSTimeInterval)time windowNumber:(int)wNum context:(NSDPSContext *)context eventNumber:(int)eNum clickCount:(int)cNum pressure:(float)pressure;
+ (NSEvent *)keyEventWithType:(NSEventType)type location:(NSPoint)location modifierFlags:(unsigned int)flags timestamp:(NSTimeInterval)time windowNumber:(int)wNum context:(NSDPSContext *)context characters:(NSString *)keys charactersIgnoringModifiers:(NSString *)ukeys isARepeat:(BOOL)flag keyCode:(unsigned short)code;
+ (NSEvent *)enterExitEventWithType:(NSEventType)type location:(NSPoint)location modifierFlags:(unsigned int)flags timestamp:(NSTimeInterval)time windowNumber:(int)wNum context:(NSDPSContext *)context eventNumber:(int)eNum trackingNumber:(int)tNum userData:(int)data;
+ (NSEvent *)otherEventWithType:(NSEventType)type location:(NSPoint)location modifierFlags:(unsigned int)flags timestamp:(NSTimeInterval)time windowNumber:(int)wNum context:(NSDPSContext *)context subtype:(short)subtype data1:(int)d1 data2:(int)d2;

@end


/* Unicodes we reserve for function keys on the keyboard,  OpenStep reserves the range 0xF700-0xF8FF for this purpose.  The availability of various keys will be system dependent. */
enum {
    NSUpArrowFunctionKey	= 0xF700,
    NSDownArrowFunctionKey	= 0xF701,
    NSLeftArrowFunctionKey	= 0xF702,
    NSRightArrowFunctionKey	= 0xF703,
    NSF1FunctionKey		= 0xF704,
    NSF2FunctionKey		= 0xF705,
    NSF3FunctionKey		= 0xF706,
    NSF4FunctionKey		= 0xF707,
    NSF5FunctionKey		= 0xF708,
    NSF6FunctionKey		= 0xF709,
    NSF7FunctionKey		= 0xF70A,
    NSF8FunctionKey		= 0xF70B,
    NSF9FunctionKey		= 0xF70C,
    NSF10FunctionKey		= 0xF70D,
    NSF11FunctionKey		= 0xF70E,
    NSF12FunctionKey		= 0xF70F,
    NSInsertFunctionKey		= 0xF710,
    NSDeleteFunctionKey		= 0xF711,
    NSHomeFunctionKey		= 0xF712,
    NSEndFunctionKey		= 0xF713,
    NSPageUpFunctionKey		= 0xF714,
    NSPageDownFunctionKey	= 0xF715,
    NSPrintScreenFunctionKey	= 0xF716,
    NSScrollLockFunctionKey	= 0xF717,
    NSPauseFunctionKey		= 0xF718,
    NSSysReqFunctionKey		= 0xF719,
    NSBreakFunctionKey		= 0xF71A,
    NSResetFunctionKey		= 0xF71B,
    NSStopFunctionKey		= 0xF71C,
    NSMenuFunctionKey		= 0xF71D,
    NSUserFunctionKey		= 0xF71E,
    NSSystemFunctionKey		= 0xF71F,
    NSPrintFunctionKey		= 0xF720,
    NSClearLineFunctionKey	= 0xF721,
    NSClearDisplayFunctionKey	= 0xF722,
    NSInsertLineFunctionKey	= 0xF723,
    NSDeleteLineFunctionKey	= 0xF724,
    NSInsertCharFunctionKey	= 0xF725,
    NSDeleteCharFunctionKey	= 0xF726,
    NSPrevFunctionKey		= 0xF727,
    NSNextFunctionKey		= 0xF728,
    NSSelectFuctionKey		= 0xF729,
    NSLastFunctionKey		= 0xF72A
};


#ifndef STRICT_OPENSTEP

enum {		/* event subtypes for NSAppKitDefined events */
    NSWindowExposedEventType		= 0,
    NSApplicationActivatedEventType	= 1,
    NSApplicationDeactivatedEventType	= 2,
    NSWindowMovedEventType 		= 4,
    NSScreenChangedEventType		= 8
};

enum {		/* event subtypes for NSSystemDefined events */
    NSPowerOffEventType			= 1
};

#endif
